﻿-- ----------------------------------------------------------------------- --
--     Moncaí's Compact Experience Bar
--     Copyright (c) 2006-2016 Moncaí
--
--     May be copied and distributed freely as long as you do not
--     change this file.
--     
-- ----------------------------------------------------------------------- --

-- Default state
CEBVisible = true;
CEBLocked = false;

local function GetColor(num)
	local red, green, blue = GetItemQualityColor(num)
	return { r = red, g = green, b = blue, }
end

-- Constants
local LibMonoFont = "Interface\\AddOns\\MoncaiCEB\\art\\AuSans.otf";
local ThousandSeparator = ","
local TipTitle = "Moncai's Compact XP Bar"
local _
local purple =  GetColor(4) -- _G.ITEM_QUALITY_COLORS[_G.ITEM_QUALITY_EPIC]
local blue = GetColor(3) --_G.ITEM_QUALITY_COLORS[_G.ITEM_QUALITY_RARE]
--~ local GUILD_LEVEL_MAX = 25
local PLAYER_LEVEL_MAX = 100;

if (GetLocale() == "deDE") then 			-- // -- deDE
	ThousandSeparator = "."
end

local function FmtBigNum(n)
        local left, num, right = string.match(n .. "", '^([^%d]*%d)(%d*)(.-)')
        return left..(num:reverse():gsub('(%d%d%d)', '%1'.. ","):reverse())..right
end

local CompactExperienceBar = CreateFrame("StatusBar","CompactExperienceBar",UIParent)
CompactExperienceBar:SetWidth(95);
CompactExperienceBar:SetHeight(50);
CompactExperienceBar:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
CompactExperienceBar:SetFrameStrata("MEDIUM");	
CompactExperienceBar:SetClampedToScreen(1);
CompactExperienceBar:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background", edgeFile="", tile=1, tileSize=10, edgeSize=10, insets={left=2, right=2, top=2, bottom=2}});
CompactExperienceBar:SetBackdropColor(0,0,0,1)
CompactExperienceBar:SetMovable(1);
CompactExperienceBar:EnableMouse(1);

CompactExperienceBar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
CompactExperienceBar:SetStatusBarColor(0,0,1)

local CEBarText = CompactExperienceBar:CreateFontString("CEBarText","OVERLAY");
CEBarText:SetFontObject(GameFontNormal);
CEBarText:SetPoint("CENTER", CompactExperienceBar, "CENTER", 0, 0);
CEBarText:SetJustifyH("CENTER");
CEBarText:SetJustifyV("CENTER");

local CompactExperienceBar1 = CreateFrame("Frame","CompactExperienceBar1",CompactExperienceBar)
CompactExperienceBar1:SetPoint("TOPLEFT", CompactExperienceBar, "TOPLEFT", -2, 1);
CompactExperienceBar1:SetPoint("BOTTOMRIGHT", CompactExperienceBar, "BOTTOMRIGHT", 2, -1);
CompactExperienceBar1:SetBackdrop({bgFile="", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=1, tileSize=10, edgeSize=10, insets={left=2, right=2, top=2, bottom=2}});
CompactExperienceBar1:SetBackdropColor(0,0,0,1)

CompactExperienceBar:RegisterEvent("PLAYER_ENTERING_WORLD");
CompactExperienceBar:RegisterEvent("VARIABLES_LOADED");
CompactExperienceBar:RegisterEvent("PLAYER_XP_UPDATE");
CompactExperienceBar:RegisterEvent("PLAYER_LOGIN");
CompactExperienceBar:RegisterEvent("PLAYER_LEVEL_UP");	

-- // Tooltip // --------------------------------------------

local CompactExperienceBarTooltip = CreateFrame( "GameTooltip", "CompactExperienceBarTooltip", nil, "GameTooltipTemplate" ); -- Tooltip name cannot be nil
CompactExperienceBarTooltip:SetOwner(CompactExperienceBar, "ANCHOR_NONE" );

CompactExperienceBar:SetScript("OnEnter", function(self, ...)
	local tip = CompactExperienceBarTooltip
	local r,g,b = _G.NORMAL_FONT_COLOR[1], _G.NORMAL_FONT_COLOR[2], _G.NORMAL_FONT_COLOR[3]
	local curxp, levelmax, rest = UnitXP("player") or 0, UnitXPMax("player") or 0, GetXPExhaustion() or 0;
	local togo = levelmax - curxp
	local perc = string.format("%.1f%%", (100*(curxp/levelmax)))
	local perbox = levelmax / 20
	
	local strcur   = string.format("%s %4.1fb %5.1f%%",  FmtBigNum(curxp), curxp / perbox, curxp/levelmax*100)
	local strtogo = string.format("%s %4.1fb %5.1f%%",  FmtBigNum(togo), togo / perbox, togo/levelmax*100)
	local strrest  = string.format("%s %4.1fb %5.1f%%",  FmtBigNum(rest), rest / perbox, rest/levelmax*100)
		
	tip:SetOwner(self, "ANCHOR_TOPRIGHT")
	tip:ClearLines();
	tip:AddLine(TipTitle, 0.1, 0.7, 0.1)
	tip:AddDoubleLine("XP", strcur, r,g,b, purple.r, purple.g, purple.b)
	tip:AddDoubleLine("To go", strtogo, r,g,b, purple.r, purple.g, purple.b)	
	tip:AddDoubleLine("Rested", strrest, r,g,b, blue.r, blue.g, blue.b)
	
--~ 	if IsInGuild() and GetGuildLevel() < GUILD_LEVEL_MAX then
--~ 		local XP_PER_QUEST = 60000
--~ 		local currentXP, remainingXP = UnitGetGuildXP("player");
--~ 		local nextLevelXP = currentXP + remainingXP;
--~ 		local nQuests = (remainingXP / XP_PER_QUEST) + 0.5
--~ 		
--~ 		if nextLevelXP > 0 then
--~ 			local strguild = string.format("%d quests (%s/%s) %5.1f%%", nQuests, FmtBigNum(currentXP), FmtBigNum(nextLevelXP), currentXP/nextLevelXP*100)

--~ 			GameTooltip:AddLine()
--~ 		
--~ 			tip:AddDoubleLine("Guild", strguild, r,g,b, 0.75,0.75,1)
--~ 		end
--~ 	end
	
	for i = 2, 5, 1 do
		_G["CompactExperienceBarTooltipTextRight" .. i]:SetFont(LibMonoFont, 12);
	end	
	tip:Show();
end)

CompactExperienceBar:SetScript("OnLeave", function(self, ...)
	CompactExperienceBarTooltip:Hide();
end)

-- // Event Handling // --------------------------------------------

CompactExperienceBar:SetScript("OnEvent", function(self, event, ...)
	local curxp = UnitXP("player")
	local levelmax = UnitXPMax("player")
	
	local mystr
	if levelmax > 0 then
		local perbox = levelmax / 20
		local perc = string.format("%.1f%%", (100*(curxp/levelmax)))
		mystr = string.format("%.1fb - %.1fb tnl | %s",  curxp / perbox, (levelmax - curxp) / perbox, perc)
	else
		mystr = "Max. level | 100%"
	end

	local r=GetXPExhaustion()
	if -1 == (r or -1) then -- no rested
		CompactExperienceBar:SetStatusBarColor(0.5,0,0.5)
	else -- rested
		CompactExperienceBar:SetStatusBarColor(0,0,1)
		--mystr = mystr .. string.format(" %.1fr",  r / perbox)
	end
	
	CEBarText:SetText(mystr)
	CompactExperienceBar:SetWidth(CEBarText:GetStringWidth()+10)
	CompactExperienceBar:SetHeight(CEBarText:GetStringHeight()+10)
	
	CompactExperienceBar:SetMinMaxValues(0, levelmax)
	CompactExperienceBar:SetValue(curxp)

	if CEBVisible then 
		CompactExperienceBar:Show() 
	else
		CompactExperienceBar:Hide()
	end
end)

-- // Make frame movable // --------------------------------------------

CompactExperienceBar:SetScript("OnMouseDown", function() 
	if not CEBLocked then 
		CompactExperienceBar:StartMoving() 
	else 
		CompactExperienceBar:StopMovingOrSizing() 
	end 
end)

CompactExperienceBar:SetScript("OnMouseUp", function() CompactExperienceBar:StopMovingOrSizing() end)	

-- // Slash command // --------------------------------------------

SLASH_CEB1 = '/ceb';
local function slashcmdhandler(param)
	if param == "hide" then
		if CEBVisible then
			CEBVisible=false
			CompactExperienceBar:Hide() 
			print("Experience hidden, type /ceb show to show.")
		end
	elseif param=="show" then
		if not CEBVisible then
			CEBVisible=true
			CompactExperienceBar:Show()
			print("Experience displayed, type /ceb hide to hide.")
		end
	elseif param=="rested" then
		if GetXPExhaustion() == nil then
			print("You have no rested EXP.")
		else
			print("You have "..GetXPExhaustion().." rested EXP remaining.")
		end
	else
		print("No command or incorrect command entered. Valid commands are:");
		print("/ceb show (Show the compact experience bar.)");
		print("/ceb hide (Hide the compact experience bar.)");
		print("/ceb rested (Shows the amount of rested experience.)");
		print("Right click the compact experience bar to lock and unlock movement.");
	end
end
SlashCmdList["CEB"] = slashcmdhandler;

-- // Quest Log functionality; alternative display // --------------------------------------------

local ReceiveSTR = ""

local function calcboxes()
	local f = QuestInfoXPFrame.ValueText -- QuestInfoXPFramePoints --TODO change this field
	--QuestInfoRewardsFrame.XPFrame.ReceiveText QuestInfo.lua: 494 thereabouts!
	local k, v , y = string.gmatch(f:GetText() or "", "(%d+),?(%d*),?(%d*)")();
	k = (k or "") .. (v or "") .. (y or "");	
	local xp = 0 + (tonumber( k ) or 0)	
	
	if xp > 0 then
		local levelmax=UnitXPMax("player")
		local perc = (xp / levelmax) * 20
		local amt = string.format("%.1f", perc)
		if amt == "0.0" then
			amt = string.format("%.2f", perc)
			if amt == "0.00" then
				amt = "0"
			end
		end
		local curxp=UnitXP("player")
		
		local result = string.format("%s +%sb %.1f%% %dx", FmtBigNum(xp), amt, (xp / levelmax) * 100, ((levelmax-curxp) / xp) + 0.5 )
		--Print("Quest has: " .. result)
		f:SetText(result)
		
--~ 		if ReceiveSTR == "" then
--~ 			ReceiveSTR = MapQuestInfoRewardsFrame.ItemReceiveText:GetText()
--~ 		end
--~ 		MapQuestInfoRewardsFrame.ItemReceiveText:SetText(ReceiveSTR .. " " .. result)
	end
end

-- Interface\AddOns\Blizzard_GuildUI\Blizzard_GuildUI.lua:408 GuildXPBar_OnEnter(self)
--~ function CEB_GuildXPBar_OnEnter(self)
--~ 	local XP_PER_QUEST = 60000
--~ 	local currentXP, remainingXP = UnitGetGuildXP("player");
--~ 	local nextLevelXP = currentXP + remainingXP;
--~ 	local nQuests = (remainingXP / XP_PER_QUEST) + 0.5
--~ 	
--~ 	--GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
--~ 	--GameTooltip:SetText(GUILD_EXPERIENCE);
--~ 	--GameTooltip:AddLine(GUILD_EXPERIENCE_TOOLTIP, 1, 1, 1, 1);
--~ 	if nextLevelXP > 0 then
--~ 	--	local percentTotal = tostring(math.ceil((currentXP / nextLevelXP) * 100));
--~ 	--	GameTooltip:AddLine(string.format(GUILD_EXPERIENCE_CURRENT, BreakUpLargeNumbers(currentXP), BreakUpLargeNumbers(nextLevelXP), percentTotal));
--~ 		GameTooltip:AddLine(string.format("%d quests to next guild level.", nQuests))
--~ 	end
--~ 	GameTooltip:Show();
--~ end

--W:\Games\World of Warcraft\BlizzardInterfaceCode 510\Interface\FrameXML\UIParent.lua:419 GuildFrame_LoadUI()
--~ local hooked = false
--~ local function guildxpc()	
--~ 	if hooked == false then
--~ 		GuildXPBar:HookScript("OnEnter", CEB_GuildXPBar_OnEnter)	
--~ 		--hooksecurefunc("GuildXPBar_OnEnter", CEB_GuildXPBar_OnEnter)
--~ 		hooked = true
--~ 	end
--~ end

local function XPHooker()
	hooksecurefunc("QuestInfo_ShowRewards", calcboxes)
--~ 	hooksecurefunc("QuestLog_UpdateQuestDetails", calcboxes) --! no longer there, needs replace
	hooksecurefunc("QuestFrameRewardPanel_OnShow", calcboxes)
	hooksecurefunc("QuestFrameDetailPanel_OnShow", calcboxes)
	hooksecurefunc("QuestInfo_Display", calcboxes)
--~ 	hooksecurefunc("GuildFrame_LoadUI", guildxpc)
	--hooksecurefunc("GuildXPBar_OnEnter", CEB_GuildXPBar_OnEnter)
end

XPHooker()

if UnitLevel("player") >= PLAYER_LEVEL_MAX then
	CompactExperienceBar:Hide() print("Experience bar hidden, type /ceb show to show.")
	CEBVisible = false
end